import React, { ChangeEvent, useMemo, useState } from "react";
import { FormControl, Grid, Input, InputLabel, Typography, FormHelperText } from "@material-ui/core";
import { DatePicker } from "@material-ui/pickers";
import { CompanyForm } from "@/forms/company";
import { StudentForm } from "@/forms/student";
import { sampleStudent } from "@/provider/dummy/student";
import { emptyInternship, Internship } from "@/data";
import { Nullable } from "@/helpers";
import moment, { Moment } from "moment";
import { computeWorkingHours } from "@/utils/date";

export type InternshipFormProps = {}

export type InternshipDetailsFormProps = {
    internship: Nullable<Internship>
}

const InternshipDurationForm = ({ internship }: InternshipDetailsFormProps) => {
    const [startDate, setStartDate] = useState<Moment | null>(internship.startDate);
    const [endDate, setEndDate] = useState<Moment | null>(internship.endDate);
    const [overrideHours, setHoursOverride] = useState<number | null>(null)
    const [workingHours, setWorkingHours] = useState<number>(40)

    const computedHours = useMemo(() => startDate && endDate && computeWorkingHours(startDate, endDate, workingHours / 5), [startDate, endDate, workingHours]);

    const hours = useMemo(() => overrideHours !== null ? overrideHours : computedHours || null, [overrideHours, computedHours]);
    const weeks = useMemo(() => hours !== null ? Math.floor(hours / 40) : null, [ hours ]);

    return (
        <Grid container>
            <Grid item md={ 4 }>
                <DatePicker value={ startDate } onChange={ setStartDate } clearable variant="inline" label={ "Data rozpoczęcia praktyki" } disableToolbar
                            fullWidth/>
            </Grid>
            <Grid item md={ 4 }>
                <DatePicker value={ endDate } onChange={ setEndDate }
                            clearable disableToolbar fullWidth
                            variant="inline" label={ "Data zakończenia praktyki" }
                            minDate={ startDate || moment() }
                />
            </Grid>
            <Grid item md={ 4 }/>
            <Grid item md={ 4 }>
                <FormControl fullWidth>
                    <InputLabel>Wymiar etatu</InputLabel>
                    <Input value={ workingHours }
                           onChange={ ev => setWorkingHours(parseInt(ev.target.value) || 0) }
                           fullWidth
                    />
                    <FormHelperText>Liczba godzin w tygodniu roboczym</FormHelperText>
                </FormControl>
            </Grid>
            <Grid item md={ 4 }>
                <FormControl fullWidth>
                    <InputLabel>Łączna liczba godzin</InputLabel>
                    <Input value={ hours || "" }
                           onChange={ ev => setHoursOverride(parseInt(ev.target.value) || 0) }
                           fullWidth
                    />
                </FormControl>
            </Grid>
            <Grid item md={ 4 }>
                <FormControl fullWidth>
                    <InputLabel>Liczba tygodni</InputLabel>
                    <Input value={ weeks || "" }
                           disabled
                           fullWidth
                    />
                </FormControl>
            </Grid>
        </Grid>
    );
}

export const InternshipForm: React.FunctionComponent<InternshipFormProps> = props => {
    const [internship, setInternship] = useState<Nullable<Internship>>(emptyInternship)

    return (
        <div className="internship-form">
            <Typography variant="h5">Dane osoby odbywającej praktykę</Typography>
            <StudentForm student={ sampleStudent }/>
            <Typography variant="h5" style={ { marginTop: "1rem" } }>Czas trwania praktyki</Typography>
            <InternshipDurationForm internship={ internship }/>
            <Typography variant="h5" style={ { marginTop: "1rem" } }>Miejsce odbywania praktyki</Typography>
            <CompanyForm/>
        </div>
    )
}

